<?php

namespace EventPortal\Http\Controllers\Admin\EventSpeaker;

use EventPortal\Contracts\EventSpeaker;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\QuickButtons\Facades\QuickButtons;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\RedirectResponse;
use Inertia\Response;

class EventSpeakerController extends Controller
{
    public function index(Event $event): Response
    {
        $query = Speaker::query()
            ->join('event_speaker', static function (JoinClause $query) {
                return $query->on('speakers.id', '=', 'event_speaker.speaker_id');
            })
            ->joinRelationship('user')
            ->where('event_speaker.event_id', '=', $event->id);

        $table = Table::make('event-speaker.index')
            ->context($event)
            ->query($query)
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Speakers')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->status(optional($event->approval)->getStatusTitleAttribute())
            ->render();

        $quickButtons = QuickButtons::make('event-speakers.index')->context($event)->get();

        return AdminRender::component('Admin/EventSpeaker/Index')
            ->key('event.speaker.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->withSideBar($sidebar)
            ->attributes(['event' => $event])
            ->withQuickButtons($quickButtons)
            ->get();
    }

    public function show(Event $event, Speaker $speaker): Response
    {
        $eventSpeaker = resolve(EventSpeaker::class)
            ->forEventAndSpeaker($event, $speaker)
            ->first();

        $speakerData = Form::make('event-speaker.show')
            ->model($eventSpeaker)
            ->context($speaker->user, $event, $speaker)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Speakers', route('admin.event.speaker.index', $event))
            ->add($speaker->user->full_name)
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/EventSpeaker/Show')
            ->key('event.speaker.show')
            ->withData($speakerData)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($speaker->name)
            ->withSideBar($sidebar)
            ->get();
    }

    public function edit(Event $event, Speaker $speaker): Response
    {
        $eventSpeaker = resolve(EventSpeaker::class)
            ->forEventAndSpeaker($event, $speaker)
            ->first();

        $form = Form::make('event-speaker.edit')
            ->model($eventSpeaker)
            ->context($speaker->user, $event, $speaker)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Speakers', route('admin.event.speaker.index', $event))
            ->add($speaker->user->full_name, route('admin.event.speaker.show', [
                'event' => $event,
                'speaker' => $speaker,
            ]))
            ->add('Edit Delivery Details')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/EventSpeaker/Edit')
            ->key('event.speaker.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($speaker->name)
            ->withSideBar($sidebar)
            ->get();
    }

    public function update(Event $event, Speaker $speaker): RedirectResponse
    {
        $eventSpeaker = resolve(EventSpeaker::class)
            ->forEventAndSpeaker($event, $speaker)
            ->firstOrFail();

        Form::make('event-speaker.edit')
            ->model($eventSpeaker)
            ->validate(request()->all())
            ->handle(request()->all());

        toast()->success(trans('event-speaker.updated'));

        return redirect()->route('admin.event.speaker.show', [
            'event' => $event,
            'speaker' => $speaker,
        ]);
    }
}
