<?php

namespace EventPortal\Tests\Unit\Util;

use Carbon\Carbon;
use EventPortal\Domain\Address\Models\Address;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\ValueObjects\EventDetails;
use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class DelegateRenderTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    /** @test **/
    public function can_add_event_details()
    {
        $event = factory(Event::class)->create(['name' => 'Event Name']);
        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => Carbon::parse('2020-05-20'),
        ]);

        $render = DelegateRender::component('Test\Component')
            ->withEventDetails(EventDetails::make($event))
            ->getData('eventDetails');

        $this->assertArraySubset([
            'name' => 'Event Name',
            'date' => '20th May 20',
        ], $render);
    }

    /** @test **/
    public function can_add_title()
    {
        $render = DelegateRender::component('Test\Component')
            ->withTitle('Page Title')
            ->getData('pageHeader');

        $this->assertEquals([
            'title' => 'Page Title',
        ], $render);
    }

    /** @test **/
    public function can_add_page_header()
    {
        $render = DelegateRender::component('Test\Component')
            ->withPageHeader(PageHeader::make()
                ->description('Description')
                ->title('Page Title')
                ->heroImage('https://www.fillmurray.com/800/450'))
            ->getData('pageHeader');

        $this->assertEquals([
            'title' => 'Page Title',
            'description' => 'Description',
            'callToAction' => null,
            'heroImage' => 'https://www.fillmurray.com/800/450'
        ], $render);
    }

    /** @test **/
    public function can_add_speakers()
    {
        $userA = factory(User::class)->create([
            'first_name' => 'Aaron',
            'last_name' => 'Andrews',
        ]);
        $speakerA = factory(Speaker::class)->create([
            'user_id' => $userA->id,
        ]);

        $userB = factory(User::class)->create([
            'first_name' => 'Betty',
            'last_name' => 'Butcher',
        ]);
        $speakerB = factory(Speaker::class)->create([
            'user_id' => $userB->id,
        ]);

        $speakers = collect([$speakerA, $speakerB]);

        $render = DelegateRender::component('Test\Component')
            ->withSpeakers($speakers)
            ->getData('speakers');

        $this->assertEquals([
            'title' => 'Speakers',
            'list' => collect([
                [
                    'name' => 'Aaron Andrews',
                    'title' => null,
                    'description' => null,
                ],
                [
                    'name' => 'Betty Butcher',
                    'title' => null,
                    'description' => null,
                ]
            ]),
        ], $render);
    }

    /** @test **/
    public function can_add_venues()
    {
        $venueA = factory(Venue::class)->state('has-address')->create([
            'name' => 'Adam\'s Accommodation',
            'website' => 'https://accommodation.com',
        ]);

        $venueB = factory(Venue::class)->state('has-address')->create([
            'name' => 'Barbara\'s B&B',
            'website' => 'https://bnb.com',
        ]);

        $render = DelegateRender::component('Test\Component')
            ->withVenues(collect([$venueA, $venueB]))
            ->getData('venues');

        $this->assertEquals([
            'title' => 'Venues',
            'list' => collect([
                [
                    'name' => 'Adam\'s Accommodation',
                    'description' => null,
                    'link' => 'https://accommodation.com',
                    'button' => null,
                    'address' => '123 Fake Street, Knotreel Borough, VA, Milan, The Shire, Plumage, DY10 4EU'
                ],
                [
                    'name' => 'Barbara\'s B&B',
                    'description' => null,
                    'link' => 'https://bnb.com',
                    'button' => null,
                    'address' => '123 Fake Street, Knotreel Borough, VA, Milan, The Shire, Plumage, DY10 4EU'
                ],
            ]),
        ], $render);
    }
}
